/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.function.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.tinygroup.function.Function;
import org.tinygroup.function.FunctionManager;
import org.tinygroup.function.FunctionManagerFactory;
import org.tinygroup.function.config.DomainDefine;
import org.tinygroup.function.config.DomainDefines;
import org.tinygroup.function.config.FunctionDefine;
import org.tinygroup.function.config.FunctionDefines;
import org.tinygroup.springutil.SpringUtil;

/**
 * 函数管理器工厂实现类<br>
 * 要先查找函数定义，再查找领域定义
 * 
 * @author luoguo
 * 
 */
public class FunctionManagerFactoryImpl implements FunctionManagerFactory {
	private static Map<String, FunctionManager> functionManagerMap = new HashMap<String, FunctionManager>();
	private static final String DEFAULT_GROUP = "";// 默认组中的函数可以在所有组中使用
	/**
	 * 分组存储<br>
	 * key:函数名,value:同名函数组成的列表
	 */
	private Map<String, List<Function<?>>> groupMap = new HashMap<String, List<Function<?>>>();
	/**
	 * 领域存储<br>
	 * key:领域名,value:领域中的函数Map,key:函数名，value，函数列表
	 */
	private Map<String, Map<String, List<Function<?>>>> domainMap = new HashMap<String, Map<String, List<Function<?>>>>();

	public synchronized FunctionManager getFunctionManager(String domainName) {
		FunctionManager functionManager = functionManagerMap.get(domainName);
		if (functionManager == null) {
			functionManager = SpringUtil.getBean(FunctionManager.class);
			Map<String, List<Function<?>>> functionMap = domainMap
					.get(domainName);
			if (functionMap != null) {
				functionManager.setFunctionMap(functionMap);
			}
			functionManagerMap.put(domainName, functionManager);
		}
		return functionManager;
	}

	public void addDomainDefines(DomainDefines domainDefines) {
		if (domainDefines.getDomainDefineList() != null) {
			for (DomainDefine domainDefine : domainDefines
					.getDomainDefineList()) {
				buildDomain(domainDefine);
			}
		}
	}

	private void buildDomain(DomainDefine domainDefine) {
		if (domainDefine.getGroupNames() != null) {
			String[] groupNameArray = domainDefine.getGroupNames().split(",");
			for (String groupName : groupNameArray) {
				buildDomain(domainDefine.getDomainName(), groupName);
			}
		}
		buildDomain(domainDefine.getDomainName(), DEFAULT_GROUP);
	}

	private void buildDomain(String domainName, String groupName) {
		List<Function<?>> functionList = groupMap.get(groupName);
		if (functionList != null) {
			Map<String, List<Function<?>>> functionMap = domainMap
					.get(domainName);
			if (functionMap == null) {
				functionMap = new HashMap<String, List<Function<?>>>();
				domainMap.put(domainName, functionMap);
			}
			addToFunctionMap(functionMap, functionList);
		}
	}

	private void addToFunctionMap(Map<String, List<Function<?>>> functionMap,
			List<Function<?>> functionList) {
		for (Function<?> function : functionList) {
			List<Function<?>> funList = functionMap.get(function.getName());
			if (funList == null) {
				funList = new ArrayList<Function<?>>();
				functionMap.put(function.getName(), funList);
			}
			funList.add(function);
		}
	}

	public void addFunctionDefines(FunctionDefines functionDefines) {
		if (functionDefines.getFunctionDefineList() != null) {
			for (FunctionDefine functionDefine : functionDefines
					.getFunctionDefineList()) {
				addToFunctionMap(functionDefines, functionDefine);
			}
		}

	}

	/**
	 * 添加函数定义到函数Map中<br>
	 * 允许函数重载
	 * 
	 * @param functionDefine
	 */
	private void addToFunctionMap(FunctionDefines functionDefines,
			FunctionDefine functionDefine) {
		String groupName = functionDefine.getGroupName();
		if (groupName == null) {
			groupName = functionDefines.getDefaultGroupName();
		}
		if (groupName == null) {
			groupName = DEFAULT_GROUP;
		}
		List<Function<?>> functionList = groupMap.get(groupName);
		if (functionList == null) {
			functionList = new ArrayList<Function<?>>();
			groupMap.put(groupName, functionList);
		}
		functionList.add((Function<?>) SpringUtil.getBean(functionDefine
				.getBeanName()));
	}

	public Set<String> getDomains() {
		return domainMap.keySet();
	}

	public Set<String> getGroups() {
		return groupMap.keySet();
	}

	public List<Function<?>> getGroupFunctions(String group) {
		return groupMap.get(group);
	}

}
